source(g_currentModDirectory.."scripts/utils.lua")

local dayNightTimescale = {};
local _dayNightTimescale_mt = Class(dayNightTimescale);
addModEventListener(dayNightTimescale);

dayNightTimescale.modName = g_currentModName
dayNightTimescale.modDirectory = g_currentModDirectory

dayNightTimescale.dayStart = 8
dayNightTimescale.nightStart = 20

dayNightTimescale.dayScale = {state = 2, value = 1}
dayNightTimescale.nightScale = {state = 11, value = 120}


function dayNightTimescale:loadMap()
    if g_server then
        g_messageCenter:subscribe(MessageType.HOUR_CHANGED, self.onHourChanged, self)
        g_messageCenter:subscribe(MessageType.SLEEPING, self.onSleeping, self)
        self:loadFromXMLFile(g_currentMission.missionInfo)
        self:onHourChanged()
    end
end

function dayNightTimescale:deleteMap()
    if g_server then
        g_messageCenter:unsubscribeAll(self)
    end
end

function dayNightTimescale:onSleeping()
    log("SLEEPING "..getDayTime())
    if g_sleepManager.isSleeping then
        g_messageCenter:unsubscribe(MessageType.MINUTE_CHANGED, self)
        self.isRunning = nil
    else
        self:onHourChanged()
    end
end

function dayNightTimescale:onHourChanged()
    if g_sleepManager.isSleeping then
        return
    end

    local env = g_currentMission.environment
    local daylight = env.daylight
    local dayTime = getDayTime()
    log("Day: "..daylight.dayStart.."-"..daylight.dayEnd.."  Night: "..daylight.nightStart.."-"..daylight.nightEnd)
    log("Hour changed: "..dayTime)

    local soonDawn, isDawn, soonDusk, isDusk = self:getDawnDusk(dayTime, daylight)

    if soonDusk or isDusk or soonDawn or isDawn then
        if self.isRunning == nil then
            g_messageCenter:subscribe(MessageType.MINUTE_CHANGED, self.onMinuteChanged, self)
            if soonDusk or isDusk then
                self.isRunning = {
                    startTime = math.max(daylight.dayEnd, dayTime),
                    endTime = daylight.nightStart,
                    startScale = -1,
                    endScale = self.nightScale.value
                }
            else
                self.isRunning = {
                    startTime = math.max(daylight.nightEnd, dayTime),
                    endTime = daylight.dayStart,
                    startScale = -1,
                    endScale = self.dayScale.value
                }
            end
            self.isRunning.duration = self.isRunning.endTime - self.isRunning.startTime
            printTable(self.isRunning, true)
        end
    elseif self.isRunning ~= nil then
        -- clear block when we've exited the lerp period
        self.isRunning = nil
    end
end

function dayNightTimescale:onMinuteChanged()
    local run = self.isRunning
    local env = g_currentMission.environment
    local missionInfo = g_currentMission.missionInfo

    -- Manual override check
    if run.prevScale and run.prevScale ~= missionInfo.timeScale then
        log("Timescale changed outside. Aborting")
        -- set to false to block it from starting again on the next hour_changed this lerp period
        self.isRunning = false
        g_messageCenter:unsubscribe(MessageType.MINUTE_CHANGED, self)
        return
    end


    local dayTime = getDayTime()
    local adjustedDayTime = dayTime - run.startTime
    -- Note: inverseLerp is clamped [0-1]
    local lerp = MathUtil.inverseLerp(run.startTime, run.endTime, dayTime)

    log("MinuteChanged: "..dayTime)
    log(adjustedDayTime.." : "..lerp)

    if run.startScale == -1 and lerp > 0 then
        run.startScale = missionInfo.timeScale
        log("Set startScale: "..run.startScale)
    end

    if lerp > 0 and lerp < 1 then
        local ts = MathUtil.lerp(run.startScale, run.endScale, lerp)
        -- round off to 1 decimal when below 1, as only the integer part is displayed above 1
        local ts_rounded = (ts < 1) and MathUtil.round(ts * 10) / 10 or MathUtil.round(ts)
        log("Timescale - raw: "..ts.." Round: "..ts_rounded)
        -- don't send timescale updates unless it's actually changed
        if ts_rounded ~= missionInfo.timeScale then
            g_currentMission:setTimeScale(ts_rounded)
        end
        run.prevScale = missionInfo.timeScale
    elseif lerp >= 1 then
        g_currentMission:setTimeScale(run.endScale)
        run = nil
        g_messageCenter:unsubscribe(MessageType.MINUTE_CHANGED, self)
    end
end

function dayNightTimescale:getDawnDusk(dayTime, daylight)
    local isDusk = dayTime > daylight.dayEnd and dayTime < daylight.nightStart
    local isDawn = dayTime > daylight.nightEnd and dayTime < daylight.dayStart
    log("Dusk: "..tostring(isDusk).." Dawn: "..tostring(isDawn))

    local soonDusk = dayTime < daylight.dayEnd and (dayTime + 1) > daylight.dayEnd
    local soonDawn = dayTime < daylight.nightEnd and (dayTime + 1) > daylight.nightEnd
    log("soonDusk: "..tostring(soonDusk).." soonDawn: "..tostring(soonDawn))

    return soonDawn, isDawn, soonDusk, isDusk
end

function dayNightTimescale:onDayNightTimescaleFieldsDayScaleChanged(state)
    log("DayScale changed: "..tostring(state))
    self.dayScale.state = state
    self.dayScale.value = Utils.getTimeScaleFromIndex(state)
end

function dayNightTimescale:onDayNightTimescaleFieldsNightScaleChanged(state)
    log("NightScale changed: "..tostring(state))
    self.nightScale.state = state
    self.nightScale.value = Utils.getTimeScaleFromIndex(state)
end


function dayNightTimescale:loadFromXMLFile(missionInfo)
    if g_server then
        if missionInfo.savegameDirectory and fileExists(missionInfo.savegameDirectory.."/DayNightTimescale.xml") then
            local xmlFile = XMLFile.load("DayNightTimescale", missionInfo.savegameDirectory.."/DayNightTimescale.xml")
            if xmlFile then
                self:onDayNightTimescaleFieldsDayScaleChanged(xmlFile:getFloat("DayNightTimescale.DayScale", self.dayScale.state))
                self:onDayNightTimescaleFieldsNightScaleChanged(xmlFile:getFloat("DayNightTimescale.NightScale", self.nightScale.state))

                xmlFile:delete()
            end

        end
    end
end

function dayNightTimescale.saveToXMLFile(missionInfo)
    if missionInfo.isValid then
        local xmlFile = XMLFile.create("DayNightTimescale", missionInfo.savegameDirectory .. "/DayNightTimescale.xml", "DayNightTimescale")
        if xmlFile ~= nil then
            xmlFile:setFloat("DayNightTimescale.DayScale", dayNightTimescale.dayScale.state)
            xmlFile:setFloat("DayNightTimescale.NightScale", dayNightTimescale.nightScale.state)
            xmlFile:save()
            xmlFile:delete()
        end
    end
end


function dayNightTimescale:installGUI()
    if not self.initDNTSGuiDone then
        self.DNTSDayScale = self.multiTimeScale:clone()
        self.DNTSDayScale.target = dayNightTimescale
        self.DNTSDayScale.id = "DNTSDayScale"
        self.DNTSDayScale:setCallback("onClickCallback", "onDayNightTimescaleFieldsDayScaleChanged")
        self.DNTSDayScale.focusId = nil
        FocusManager:loadElementFromCustomValues(self.DNTSDayScale, nil, nil, false, false)

        self.DNTSDayScale.elements[4]:setText(g_i18n.modEnvironments.FS22_DayNightTimeScale:getText("Setting_DayScale"))
        self.DNTSDayScale.elements[6]:setText(g_i18n.modEnvironments.FS22_DayNightTimeScale:getText("Explanation_DayScale"))

        self.DNTSNightScale = self.multiTimeScale:clone()
        self.DNTSNightScale.target = dayNightTimescale
        self.DNTSNightScale.id = "DNTSNightScale"
        self.DNTSNightScale:setCallback("onClickCallback", "onDayNightTimescaleFieldsNightScaleChanged")
        self.DNTSNightScale.focusId = nil
        FocusManager:loadElementFromCustomValues(self.DNTSNightScale, nil, nil, false, false)

        self.DNTSNightScale.elements[4]:setText(g_i18n.modEnvironments.FS22_DayNightTimeScale:getText("Setting_NightScale"))
        self.DNTSNightScale.elements[6]:setText(g_i18n.modEnvironments.FS22_DayNightTimeScale:getText("Explanation_NightScale"))


        local title = TextElement.new()
        title:applyProfile("settingsMenuSubtitle", true)
        title:setText(g_i18n.modEnvironments.FS22_DayNightTimeScale:getText("Title_DayNightTimescale"))

        self.boxLayout:addElement(title)
        self.boxLayout:addElement(self.DNTSDayScale)
        self.boxLayout:addElement(self.DNTSNightScale)

        self.initDNTSGuiDone = true

        self.DNTSDayScale:setState(dayNightTimescale.dayScale.state)
        self.DNTSNightScale:setState(dayNightTimescale.nightScale.state)

        self.boxLayout:invalidateLayout()
    end
end

function dayNightTimescale:updateDayNightFieldsGUI()
    if self.initDNTSGuiDone then
        if self.DNTSDayScale then
            self.DNTSDayScale:setState(dayNightTimescale.dayScale.state)
        end
        if self.DNTSNightScale then
            self.DNTSNightScale:setState(dayNightTimescale.nightScale.state)
        end
    end
end


InGameMenuGameSettingsFrame.onFrameOpen = Utils.appendedFunction(InGameMenuGameSettingsFrame.onFrameOpen, dayNightTimescale.installGUI)
InGameMenuGameSettingsFrame.updateGameSettings = Utils.appendedFunction(InGameMenuGameSettingsFrame.updateGameSettings, dayNightTimescale.updateDayNightFieldsGUI)	
FSCareerMissionInfo.saveToXMLFile = Utils.appendedFunction(FSCareerMissionInfo.saveToXMLFile, dayNightTimescale.saveToXMLFile);